package com.olympya.tools.grok;

/**
 * A checked exception thrown when a particular {@linkplain Artifact artifact} cannot be found.
 *
 * @author Paulo Mattos
 */
public class NoSuchArtifactException
extends Exception
{
  private final Repository.Type repoType;
  private final String artifactName;

  /**
   * Constructs a {@code NoSuchArtifactException} with the specified repository and artifact info.
   */
  public NoSuchArtifactException(Repository.Type repoType, String artifactName)
  {
    super(String.format("Missing %s artifact %s", repoType, artifactName));

    if (repoType == null)
      throw new NullPointerException("repoType");

    if (artifactName == null)
      throw new NullPointerException("artifactName");

    this.repoType = repoType;
    this.artifactName = artifactName;
  }

  /**
   * Returns the repository type of the missing artifact.
   */
  public Repository.Type getRepositoryType()
  {
    return repoType;
  }

  /**
   * Returns the name of the missing artifact.
   */
  public String getArtifactName()
  {
    return artifactName;
  }
}
