package com.olympya.tools.grok;

import java.util.*;

/**
 * A repository of <em>software artifacts</em> {@linkplain Artifact.Revision revisions}.
 *
 * @author Paulo Mattos
 */
public abstract class Repository
{
  private final Type type;

  /**
   * Constructor designed for subclassing.
   *
   * @throws NullPointerException if {@code type} is {@code null}.
   */
  protected Repository(Type type)
  {
    if (type == null)
      throw new NullPointerException("type");

    this.type = type;
  }

  /**
   * Returns repository type.
   */
  public final Type getType()
  {
    return type;
  }

  //
  // Helper methods
  //

  /**
   *
   */
  public static Modification numericModification(Developer author, Date date, long id)
  {
    return new NumericModification(author, date, id);
  }

  /**
   * A repository type.
   */
  public enum Type
  {
    /**
     * A {@linkplain VersionControl version control} repository.
     */
    VERSION_CONTROL(VersionControl.class, Source.class),

    /**
     * A {@linkplain IssueTracker issue tracking} repository.
     */
    ISSUE_TRACKER(IssueTracker.class, Issue.class);

    private final Class<? extends Repository> repositoryClass;
    private final Class<? extends Artifact> artifactClass;

    private Type(Class<? extends Repository> repositoryClass, Class<? extends Artifact> artifactClass)
    {
      assert repositoryClass != null && artifactClass != null;

      this.repositoryClass = repositoryClass;
      this.artifactClass   = artifactClass;
    }

    /**
     *
     */
    public Class<? extends Repository> getRepositoryClass()
    {
      return repositoryClass;
    }

    /**
     *
     */
    public Class<? extends Artifact> getArtifactClass()
    {
      return artifactClass;
    }
  }

  /**
   * An atomic repository modification (e.g., a VCS <em>changes list</em>).
   */
  public static abstract class Modification
  implements Comparable<Modification>
  {
    /**
     *
     */
    public abstract Developer getAuthor();

    /**
     *
     */
    public abstract Date getDate();

    /**
     *
     */
    public Number toNumber()
    {
      return getDate().getTime();
    }
  }
}


