package com.olympya.tools.grok;

import java.util.*;
import static java.util.Collections.*;

import static com.google.common.collect.Maps.*;
import static com.google.common.collect.Sets.*;

/**
 * A development team, incluiding all used {@linkplain Repository repositories}.
 * <p>
 *
 * Instances of this class are immutable.
 *
 * @author Paulo Mattos
 */
public final class Team
{
  private final Set<Developer> developers;
  private final Map<Repository.Type, Repository> repositories;

  /**
   *
   */
  public Team(Collection<Developer> developers, Repository... repositories)
  {
    Map<Repository.Type, Repository> repoMap = newEnumMap(Repository.Type.class);

    for (Repository repo : repositories)
      repoMap.put(repo.getType(), repo);

    this.developers   = immutableSet(developers);
    this.repositories = unmodifiableMap(repoMap);
  }


  /**
   *
   */
  public Set<Developer> getDevelopers()
  {
    return developers;
  }

  /**
   *
   */
  public boolean hasRepository(Repository.Type repoType)
  {
    return repositories.containsKey(repoType);
  }

  /**
   *
   */
  public Repository getRepository(Repository.Type repoType)
  {
    if (!repositories.containsKey(repoType))
      throw new IllegalArgumentException("repoType=" + repoType);

    return repositories.get(repoType);
  }
}
