package com.olympya.tools.grok.intellij;

import java.util.*;
import java.net.*;

import static com.google.common.collect.Lists.newArrayList;

import com.olympya.tools.grok.*;
import com.olympya.tools.grok.intellij.ui.*;

/**
 * Issue Tracker abstraction.
 *
 * @author Hugo Baés
 */
public abstract class AbstractIssueTracker
{
  private final String name;
  private final URI vendorUrl;

  /**
   * Creates an Issue Tracker.
   */
  public AbstractIssueTracker(String name, URI vendorUrl)
  {
    this.name      = name;
    this.vendorUrl = vendorUrl;
  }

  /**
   * Returns a list of "registered" issue trackers.
   */
  public static List<AbstractIssueTracker> getAllIssueTrackers()
  {
    List<AbstractIssueTracker> trackers = newArrayList();
    trackers.add(new Bugzilla());
    trackers.add(new FogBugz());
    trackers.add(new Jira());
    return trackers;
  }

  /**
   * The name of the issue tracker.
   */
  public String getDisplayName()
  {
    return name;
  }

  /**
   * The vendor url.
   */
  public URI getVendorUrl()
  {
    return vendorUrl;
  }

  /**
   * Get IssueTracker from name.
   */
  public static AbstractIssueTracker getIssueTrackerFromName(String name)
  {
    AbstractIssueTracker tracker = null;
    for(AbstractIssueTracker t : getAllIssueTrackers())
    {
      if (t.getDisplayName().equals(name))
      {
        tracker = t;
        break;
      }
    }
    return tracker;
  }

  /**
   * Submits a but report.
   */
  public abstract void submitBugReport(Developer developer, Query query);

  /**
   * Submits a bug request.
   */
  public abstract void submitFeatureRequest(Developer developer, Query query);
}
