package com.olympya.tools.grok.intellij;

import java.net.*;

import com.intellij.ide.*;

import com.olympya.tools.grok.*;
import com.olympya.tools.grok.intellij.ui.*;

/**
 * Bugzilla issue tracker.
 *
 * @author Hugo Baés
 */
final class Bugzilla
        extends AbstractIssueTracker
{
  public Bugzilla()
  {
    super("Bugzilla", URI.create("http://www.bugzilla.org"));
  }

  /**
   * {@inheritDoc}
   */
  public void submitBugReport(Developer developer, Query query)
  {
    URI root = query.workgroup.getRepository(Repository.Type.ISSUE_TRACKER).getRoot();

    String scheme    = root.getScheme();
    String authority = root.getAuthority();
    String path      = root.getPath();
    try
    {
      URI url = new URI(scheme, authority, path + "/enter_bug.cgi");
      BrowserUtil.launchBrowser(url.toASCIIString());
    }
    catch (URISyntaxException e)
    {
      BrowserUtil.launchBrowser(root + "/enter_bug.cgi");
    }
  }

  /**
   * {@inheritDoc}
   */
  public void submitFeatureRequest(Developer developer, Query query)
  {
    URI root = query.workgroup.getRepository(Repository.Type.ISSUE_TRACKER).getRoot();

    String scheme    = root.getScheme();
    String authority = root.getAuthority();
    String path      = root.getPath();
    try
    {
      URI url = new URI(scheme, authority, path + "/enter_bug.cgi");
      BrowserUtil.launchBrowser(url.toASCIIString());
    }
    catch (URISyntaxException e)
    {
      BrowserUtil.launchBrowser(root + "/enter_bug.cgi");
    }
  }
}
