package com.olympya.tools.grok.intellij;

import java.net.*;

import com.intellij.ide.*;

import com.olympya.tools.grok.*;
import com.olympya.tools.grok.intellij.ui.*;

/**
 * FogBugz issue tracker.
 * <br/>
 * Fields can be passed by GET.
 *
 * @author Hugo Baés
 */
final class FogBugz
        extends AbstractIssueTracker
{
  public FogBugz()
  {
    super("FogBugz", URI.create("http://www.fogbugz.com"));
  }

  /**
   * {@inheritDoc}
   */
  public void submitBugReport(Developer developer, Query query)
  {
    String filename = query.artifact.getArtifact().getName();
    String filepath = "/no/path/to/file/.../yet"; //TODO: context.artifact.getLocation();
    String revision = String.valueOf(query.artifact.getModification().toNumber());
    String date     = query.artifact.getModification().getDate().toString();

    String title = "Fix bug on " + filename;
    String body  = "Path: " + filepath +" \n" +
                   "Revision: " + revision + "\n" +
                   "Date: " + date + "\n" +
                   "\n";

    URI root = query.workgroup.getRepository(Repository.Type.ISSUE_TRACKER).getRoot();

    String scheme    = root.getScheme();
    String authority = root.getAuthority();
    String path      = root.getPath();
    try
    {
      URI url = new URI(scheme, authority, path + "/default.php", "command=new&pg=pgEditBug&ixCategory=1&sTitle=" + title + "&sEvent=" + body, null);
      BrowserUtil.launchBrowser(url.toString());
    }
    catch (URISyntaxException e)
    {
      BrowserUtil.launchBrowser(root + "/default.php?command=new&pg=pgEditBug&ixCategory=1&sTitle=" + title);
    }
  }

  /**
   * {@inheritDoc}
   */
  public void submitFeatureRequest(Developer developer, Query query)
  {
    String filename = query.artifact.getArtifact().getName();
    String filepath = "/no/path/to/file/.../yet"; //TODO: context.artifact.getLocation();
    String revision = String.valueOf(query.artifact.getModification().toNumber());
    String date     = query.artifact.getModification().getDate().toString();

    String title = "New feature for " + filename;
    String body  = "Path: " + filepath +" \n" +
                   "Revision: " + revision + "\n" +
                   "Date: " + date + "\n" +
                   "\n";

    URI root = query.workgroup.getRepository(Repository.Type.ISSUE_TRACKER).getRoot();

    String scheme    = root.getScheme();
    String authority = root.getAuthority();
    String path      = root.getPath();
    try
    {
      URI url = new URI(scheme, authority, path + "/default.php", "command=new&pg=pgEditBug&ixCategory=2&sTitle=" + title + "&sEvent=" + body, null);
      BrowserUtil.launchBrowser(url.toString());
    }
    catch (URISyntaxException e)
    {
      BrowserUtil.launchBrowser(root + "/default.php?command=new&pg=pgEditBug&ixCategory=2&sTitle=" + title);
    }
  }
}
