package com.olympya.tools.grok.intellij;

import java.net.*;

import com.intellij.ide.*;

import com.olympya.tools.grok.*;
import com.olympya.tools.grok.intellij.ui.*;

/**
 * JIRA issue tracker.
 * <br/>
 * http://confluence.atlassian.com/display/JIRA/Creating+Issues+via+direct+HTML+links
 *
 * @author Hugo Baés
 */
final class Jira
        extends AbstractIssueTracker
{
  public Jira()
  {
    super("JIRA", URI.create("http://www.atlassian.com/software/jira"));
  }

  /**
   * {@inheritDoc}
   */
  public void submitBugReport(Developer developer, Query query)
  {
    URI root = query.workgroup.getRepository(Repository.Type.ISSUE_TRACKER).getRoot();

    String scheme    = root.getScheme();
    String authority = root.getAuthority();
    String path      = root.getPath();
    try
    {
      URI url = new URI(scheme, authority, path + "/secure/CreateIssue.jspa", "issuetype=0", null);
      BrowserUtil.launchBrowser(url.toASCIIString());
    }
    catch (URISyntaxException e)
    {
      BrowserUtil.launchBrowser(root + "/secure/CreateIssue.jspa?issuetype=0");
    }
  }

  /**
   * {@inheritDoc}
   */
  public void submitFeatureRequest(Developer developer, Query query)
  {
    URI root = query.workgroup.getRepository(Repository.Type.ISSUE_TRACKER).getRoot();

    String scheme    = root.getScheme();
    String authority = root.getAuthority();
    String path      = root.getPath();
    try
    {
      URI url = new URI(scheme, authority, path + "/secure/CreateIssue.jspa", "issuetype=2", null);
      BrowserUtil.launchBrowser(url.toASCIIString());
    }
    catch (URISyntaxException e)
    {
      BrowserUtil.launchBrowser(root + "/secure/CreateIssue.jspa?issuetype=2");
    }
  }
}
