package com.olympya.tools.grok.intellij;

import com.olympya.tools.grok.*;
import com.olympya.tools.grok.metrics.*;
import static com.olympya.tools.grok.Repository.Type.*;

/**
 * Some predefined {@linkplain Workgroup workgroups} useful for testing purposes.
 *
 * @author Paulo Mattos
 */
public final class PredefinedWorkgroups
{
  /**
   * Returns the Olympya workgroup (5 developers).
   */
  public static Workgroup olympya()
  {
    Workgroup.Builder wgroup = new Workgroup.Builder();

    wgroup.name("Olympya IT");

    RepositoryInfo svn = new RepositoryInfo("Subversion", "svn://svn.olympya.com");
    wgroup.repository(VERSION_CONTROL, svn);

    RepositoryInfo fogbugz = new RepositoryInfo("FogBugz", "http://fogbugz.olympya.com");
    wgroup.repository(ISSUE_TRACKER, fogbugz);

    wgroup.metric(VERSION_CONTROL, new WeightedChangesMetric(5));
    wgroup.metric(ISSUE_TRACKER,   new RandomMetric());

    wgroup.developer(dev("Paulo Mattos",  "olympya.com"));
    wgroup.developer(dev("Hugo Baes",     "olympya.com"));
    wgroup.developer(dev("Rafael Silva",  "olympya.com"));
    wgroup.developer(dev("Paulo Andre",   "olympya.com"));
    wgroup.developer(dev("Frodo Baggins", "olympya.com"));

    return wgroup.build();
  }

  private static Developer dev(String name, String domain)
  {
    String lname = name.toLowerCase();
    String email = lname.replace(' ', '.') + "@" + domain;
    String user  = lname.charAt(0) + lname.substring(lname.indexOf(' ') + 1);

    Developer.Builder dev = new Developer.Builder();

    return dev.name(name).email(email).im(user).userNames(VERSION_CONTROL, user).build();
  }

  /**
   * Driver for simple tests.
   */
  public static void main(String... args)
  {
    System.out.println(olympya());
  }
}
