package com.olympya.tools.grok.intellij.ui;

import java.awt.*;
import javax.swing.*;

/**
 * ComboBox with no selection message.
 *
 * @author Hugo Baés
 */
public class ComboBox
  extends JComboBox
{
  private String emptyText;
  private Font   emptyFont;
  private Color  emptyForeground;

  /**
   * Empty ComboBox.
   */
  public ComboBox()
  {
    super();
    defaultEmptyFormatting();
  }

  /**
   * Combo Box with choices.
   */
  public ComboBox(Object items[])
  {
    super(items);
    defaultEmptyFormatting();
  }

  // Sets empty formatting
  private void defaultEmptyFormatting()
  {
    Font defaultFont = UIManager.getFont("ComboBox.font");
    emptyText        = "(value not selected)";
    emptyFont        = new Font(defaultFont.getName(), defaultFont.getStyle() | Font.ITALIC, defaultFont.getSize());
    emptyForeground  = Color.LIGHT_GRAY;
  }

  public void setEmptyText(String t)
  {
    emptyText = t;
    repaint();
  }

  public String getEmptyText()
  {
    return emptyText;
  }

  public void setEmptyFont(Font f)
  {
    emptyFont = f;
    repaint();
  }

  public Font getEmptyFont()
  {
    return emptyFont;
  }

  public void setEmptyForeground(Color fg)
  {
    emptyForeground = fg;
    repaint();
  }

  public Color getEmptyForeground()
  {
    return emptyForeground;
  }

  @Override
  protected void paintComponent(Graphics g)
  {
    super.paintComponent(g);

    if (getSelectedIndex() == -1)
    {
      Color oldColor = g.getColor();
      Font  oldFont  = g.getFont();

      g.setColor(getEmptyForeground());
      g.setFont(getEmptyFont());
      int fontHeight = g.getFontMetrics(getEmptyFont()).getHeight();
      int y = getHeight() - fontHeight / 2;
      g.drawString(getEmptyText(), getInsets().left+2, y+1);

      g.setColor(oldColor);
      g.setFont(oldFont);
    }
  }
}
