package com.olympya.tools.grok.intellij.ui;

import org.jetbrains.annotations.NotNull;

import javax.swing.*;

/**
 * Represents an developer in popup menu.
 *
 * @author Hugo Baés
 */
final class DeveloperPopupItem
        implements PopupItem
{

  private final String developer;
  private final int expertise;
  private final Icon icon;
  private final PopupStep actions;

  /**
   * Developer with actions submenu.
   */
  public DeveloperPopupItem(String developer, int expertise, Icon icon, PopupStep actions)
  {
    this.developer = developer;
    this.expertise = expertise;
    this.icon      = icon;
    this.actions   = actions;
  }

  /**
   * Returns the developer name.
   */
  public String getDeveloper()
  {
    return developer;
  }

  /**
   * Returns the expertise level.
   */
  public int getExpertise()
  {
    return expertise;
  }

  /**
   * {@inheritDoc}
   */
  public boolean hasSubstep()
  {
    return actions != null && !actions.values.isEmpty();
  }

  /**
   * {@inheritDoc}
   */
  public PopupStep getSubstep()
  {
    return actions;
  }

  /**
   * {@inheritDoc}
   */
  public void execute()
  {
    assert false;
  }

  /**
   * {@inheritDoc}
   */
  public String getText()
  {
    return "<html>"+ developer + " <font color='gray'>(" + String.valueOf(expertise) + "%)</font></html>";
  }

  /**
   * {@inheritDoc}
   */
  public Icon getIcon()
  {
    return icon;
  }

  /**
   * {@inheritDoc}
   */
  public boolean hasSeparatorAbove()
  {
    return false;
  }

  /**
   * {@inheritDoc}
   */
  public boolean isEnabled()
  {
    return true;
  }

  /**
   * {@inheritDoc}
   */
  public boolean showSubstepOnExecution()
  {
    return true;
  }

  @NotNull @Override
  public String toString()
  {
    return developer + " (" + String.valueOf(expertise) + "%)";
  }
}
