package com.olympya.tools.grok.intellij.ui;

import java.awt.image.*;
import java.awt.*;
import java.awt.color.*;
import javax.swing.*;

import com.intellij.openapi.util.*;

/**
 * Stores all plugin's icons.
 *
 * @author Paulo Mattos
 * @author Hugo Baés
 */
public final class Icons
{
  /**
   * Grok logo icon.
   */
  public static final Icon GROK_LOGO = get("/resource-files/workgroup-big.png");

  /**
   * Workgroup icon.
   */
  public static final Icon WORKGROUP = get("/resource-files/workgroup.png");

  /**
   * Developer icon.
   */
  public static final Icon DEVELOPER = get("/resource-files/developer.png");

  /**
   * Starred developer icon.
   */
  public static final Icon STAR_DEVELOPER = get("/resource-files/star-developer.png");

  /**
   * Unknown developer icon.
   */
  public static final Icon DEV_UNKNOWN = grayscale(get("/resource-files/developer.png"));

  /**
   * Edit developer icon.
   */
  public static final Icon EDIT_DEV = get("/actions/editSource.png");

  /**
   * Email icon.
   */
  public static final Icon EMAIL = get("/resource-files/email.png");

  /**
   * IDEtalk icon.
   */
  public static final Icon IDETALK = get("/ideTalk/user.png");

  /**
   * Bug report icon.
   */
  public static final Icon BUG_REPORT = get("/resource-files/bug.png");

  /**
   * Feature request icon.
   */
  public static final Icon FEATUER_REQUEST = get("/actions/createFromUsage.png");

  /**
   * Add icon.
   */
  public static final Icon ADD = get("/actions/include.png");

  /**
   * Remove icon.
   */
  public static final Icon EXCLUDE = get("/actions/exclude.png");

  /**
   * Error icon.
   */
  public static final Icon ERROR = get("/compiler/error.png");

  /**
   * VCS Icon.
   */
  public static final Icon VCS = get("/general/vcsSmallTab.png");

  /**
   * Settings.
   */
  public static final Icon SETTINGS = get("/general/applicationSettings.png");

  private static Icon get(String iconPath)
  {
    assert iconPath != null;
    return IconLoader.getIcon(iconPath);
  }

  private static Icon grayscale(Icon icon)
  {
    assert icon != null;

    // Create blank image buffer
    final BufferedImage image = new BufferedImage(icon.getIconWidth(), icon.getIconHeight(), BufferedImage.TYPE_INT_ARGB);
    final Graphics2D graphics = image.createGraphics();

    // Paint in a fake component using image graphics
    graphics.setColor(new Color(0, 0, 0, 0)); // Transparent
    graphics.fillRect(0, 0, icon.getIconWidth(), icon.getIconHeight());
    icon.paintIcon(new JLabel(), graphics, 0, 0); // "canvas" from a dummy component
    graphics.dispose();

    // Apply image filter
    ColorSpace colorSpace = ColorSpace.getInstance(ColorSpace.CS_GRAY);
    ColorConvertOp toGray = new ColorConvertOp(colorSpace, null);
    BufferedImage grayImage = toGray.filter(image, null);
    return new ImageIcon(grayImage);
  }

  /**
   * No instances are allowed.
   */
  private Icons()
  {
    assert false;
  }
}
