package com.olympya.tools.grok.intellij.ui;

import org.jetbrains.annotations.*;
import com.intellij.openapi.application.ApplicationManager;
import com.intellij.openapi.ui.popup.util.BaseListPopupStep;
import com.intellij.openapi.ui.popup.*;

import javax.swing.*;
import java.util.*;

/**
 * Popup menu step.
 *
 * @author Hugo Baés
 */
final class PopupStep
extends BaseListPopupStep<PopupItem>
{
  List<PopupItem> values = new ArrayList<PopupItem>();

  /**
   * Popup step without name.
   */
  public PopupStep(List<PopupItem> values)
  {
    this(null, values);
  }

  /**
   * Popup step with name.
   */
  public PopupStep(@Nullable String title, List<PopupItem> values)
  {
    super(title, values);
    this.values = values;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public PopupStep onChosen(final PopupItem selectedValue, boolean finalChoice)
  {
    if (selectedValue.hasSubstep() && (!finalChoice || selectedValue.showSubstepOnExecution()))
    {
      return selectedValue.getSubstep();
    }
    else
    {
      ApplicationManager.getApplication().invokeLater(new Runnable()
      {
        public void run()
        {
          selectedValue.execute();
        }
      });
      return null;
    }
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean hasSubstep(PopupItem selectedValue)
  {
    return selectedValue.hasSubstep();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean isMnemonicsNavigationEnabled()
  {
    return false;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean isSpeedSearchEnabled()
  {
    return true;
  }

  /**
   * {@inheritDoc}
   */
  @Override  @NotNull
  public String getTextFor(PopupItem value)
  {
    return value.getText();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public Icon getIconFor(PopupItem value)
  {
    return value.getIcon();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean isAutoSelectionEnabled()
  {
    return false;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean isSelectable(PopupItem value)
  {
    return value.isEnabled();
  }

  @Override
  public ListSeparator getSeparatorAbove(PopupItem value)
  {
    if (value.hasSeparatorAbove())
      return new ListSeparator();

    return null;
  }
}
