package com.olympya.tools.grok.intellij.ui;

import java.util.*;

import com.intellij.openapi.actionSystem.*;

import com.olympya.tools.grok.*;
import com.olympya.tools.grok.intellij.*;
import com.olympya.tools.grok.metrics.*;
import static com.olympya.tools.grok.Repository.Type.*;

/**
 * Queries the expert developers set for the currently active/selected file (aka <em>grokkers</em>).
 *
 * @author Hugo Baés
 * @author Paulo Mattos
 */
public class QueryExpertOnSource
extends AnAction
{
  /**
   * Executes query.
   */
  public void actionPerformed(AnActionEvent event)
  {
    UIContext context = new UIContext(event.getDataContext());

    try
    {
      Workgroup wgroup = Plugin.getInstance().getWorkgroupFor(context.project, context.vfile);

      Source src = wgroup.getVersionControl(context.project).asSource(context.vfile);
      Team team  = wgroup.asTeam(context.project);

      Metric metric = wgroup.getMetric(VERSION_CONTROL);
      metric = metric.timed().sorted().relativized();

      UIs.showStatus(context.project, "Computing metric over '" + src.getName() + "'...");
      Map<Developer, Expertise> experts = metric.evaluate(team, src);
      UIs.showStatus(context.project, null);

      Query query = new Query(wgroup, experts, src.latestRevision(), context);

      UIs.showDevelopersPopup("Developers", query, event.getDataContext());
    }
    catch (NoSuchWorkgroupException ex)
    {
      UIs.showMessagePopup(null, ex.getMessage(), Icons.ERROR);
      UIs.showStatus(context.project, ex.getMinorMessage());
    }
    catch (NoSuchArtifactException ex)
    {
      UIs.showMessagePopup(null, ex.getArtifactName() + " is not under version control.", Icons.ERROR);
    }
  }
}