package com.olympya.tools.grok.intellij.ui;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
 * TextField with empty text.
 *
 * @author Hugo Baés
 */
public class TextField
  extends JTextField
{
  private String  emptyText;
  private Font    emptyFont;
  private Color   emptyForeground;
  private Color   emptyBackground;

  /**
   * Empty TextField.
   */
  public TextField()
  {
    this(null);
  }

  /**
   * TextField with inital text.
   */
  public TextField(String t)
  {
    super(t);

    // Set empty formatting
    Font defaultFont = UIManager.getFont("TextField.font");
    emptyText        = "(value not set)";
    emptyFont        = new Font(defaultFont.getName(), defaultFont.getStyle() | Font.ITALIC, defaultFont.getSize());
    emptyForeground  = Color.LIGHT_GRAY;
    emptyBackground  = getBackground();
  }

  public void setEmptyText(String t)
  {
    emptyText = t;
    repaint();
  }

  public String getEmptyText()
  {
    return emptyText;
  }

  public void setEmptyFont(Font f)
  {
    emptyFont = f;
    repaint();
  }

  public Font getEmptyFont()
  {
    return emptyFont;
  }

  public void setEmptyForeground(Color fg)
  {
    emptyForeground = fg;
    repaint();
  }

  public Color getEmptyForeground()
  {
    return emptyForeground;
  }

  public void setEmptyBackground(Color bg)
  {
    emptyBackground = bg;
    repaint();
  }

  public Color getEmptyBackground()
  {
    return emptyBackground;
  }

  @Override
  protected void processFocusEvent(FocusEvent e)
  {
    super.processFocusEvent(e);
    repaint();
  }

  @Override
  protected void paintComponent(Graphics g)
  {
    super.paintComponent(g);

    if (getText().length()==0 && !hasFocus())
    {
      Color oldColor = g.getColor();
      Font  oldFont  = g.getFont();

      if (isOpaque()) {
          g.setColor(getEmptyBackground());
          g.fillRect(0, 0, getWidth(), getHeight());
      }

      g.setColor(getEmptyForeground());
      g.setFont(getEmptyFont());
      int fontHeight = g.getFontMetrics(getEmptyFont()).getHeight();
      int y = getHeight() - fontHeight / 2;
      g.drawString(getEmptyText(), getMargin().left+2, y + getMargin().top+1);

      g.setColor(oldColor);
      g.setFont(oldFont);
    }
  }
}