package com.olympya.tools.grok.intellij.ui;

import java.util.*;
import javax.swing.*;
import static java.util.Collections.*;

import com.intellij.ide.*;
import com.intellij.openapi.ui.popup.*;
import com.intellij.openapi.project.*;
import com.intellij.openapi.wm.*;
import com.intellij.openapi.actionSystem.*;

import static com.google.common.collect.Lists.*;

import com.olympya.tools.grok.*;
import com.olympya.tools.grok.intellij.ui.conf.*;
import com.olympya.tools.grok.metrics.*;

import org.jetbrains.annotations.*;

/**
 * User interface components factory and helper class.
 *
 * @author Hugo Baés
 */
public final class UIs
{
  /**
   * Helper class (i.e., no instances are allowed).
   */
  private UIs()
  {
    assert false;
  }

  /**
   * Shows a popup menu based on the specified map of developers.
   */
  public static void showDevelopersPopup(String title, Query query, DataContext xxx)
  {
    // Create developers popup menu
    List<PopupItem> developerList = newArrayList();

    for (Map.Entry<Developer, Expertise> item : query.experts.entrySet())
    {
      Developer developer = item.getKey();
      Expertise expertise = item.getValue();

      // Create actions popup menu
      List<PopupItem> actionList = newArrayList();
      for (Send send : Send.values())
        actionList.add(new SendPopupItem(send, developer, query));
      actionList.add(new EditDeveloperPopupItem(developer, query));

      // Select icon based on percentage value
      Icon icon;
      if (!query.workgroup.getDevelopers().contains(developer))
        icon = Icons.DEV_UNKNOWN;
      else if (expertise.percentageValue() > 80)
        icon = Icons.STAR_DEVELOPER;
      else
        icon = Icons.DEVELOPER;

      developerList.add(new DeveloperPopupItem(developer.toString(), expertise.percentageValue(), icon, new PopupStep(actionList)));
    }

    JBPopup popup = JBPopupFactory.getInstance().createListPopup(new PopupStep(title, developerList));
    popup.showInBestPositionFor(DataManager.getInstance().getDataContext());
//    popup.showInBestPositionFor(xxx);
  }

  /**
   * Shows a popup menu containing just the specified message.
   */
  public static void showMessagePopup(@Nullable String title, String msg, Icon icon)
  {
    JBPopup popup = createMessagePopup(title, msg, icon);
    popup.showInBestPositionFor(DataManager.getInstance().getDataContext());
  }

  /**
   * Create a popup menu containing just the specified message.
   */
  public static JBPopup createMessagePopup(@Nullable String title, String msg, Icon icon)
  {
    PopupItem msgItem = new MessagePopupItem(msg, icon);
    List<PopupItem> oneItemList = singletonList(msgItem);

    return JBPopupFactory.getInstance().createListPopup(new PopupStep(title, oneItemList));
  }

  /**
   * Create a popup with add options for the active configuration form.
   */
  public static JBPopup createConfigAddPopup(ConfigurationDialog dialog)
  {
    List<PopupItem> addOptions = newArrayList();
    addOptions.add(new AddWorkgroupPopupItem(dialog));
    addOptions.add(new AddDeveloperPopupItem(dialog));

    return JBPopupFactory.getInstance().createListPopup(new PopupStep(addOptions));
  }

  /**
   * Shows the specified message in the status bar.
   */
  public static void showStatus(Project project, String message)
  {
    StatusBar statusbar = WindowManager.getInstance().getStatusBar(project);
    statusbar.setInfo(message);
  }
}