package com.olympya.tools.grok.intellij.ui.conf;

import java.awt.*;
import java.awt.event.*;
import java.util.regex.*;
import javax.swing.*;
import javax.swing.event.*;

import com.intellij.uiDesigner.core.*;

import com.olympya.tools.grok.*;
import com.olympya.tools.grok.intellij.ui.*;

/**
 * Panel with developer info.
 *
 * @author Hugo Baés
 */
class DeveloperPanel
        extends JPanel
{
  // Package-private components
  JTextField name;
  JTextField email;
  JTextField talkUsername;
  JTextField repoUsername;

  private JPanel thisPanel;
  private final String repoDefaultName = "Repository";
  private final Pattern repoPlaceholder = Pattern.compile("\\$current\\$");
  private final String repoLabelContent = "$current$ username";
  private final String repoDescriptionContent = "The username the appears on the author field of the $current$";

  private JLabel repoLabel;
  private JTextArea repoDescription;

  DeveloperPanel(final DeveloperTreeNode developer)
  {
    $$$setupUI$$$();

    DocumentListener modificationNotifier = new DocumentListener()
    {
      public void insertUpdate(DocumentEvent e)
      {
        developer.getParent().getParentModel().modified();
      }
      public void removeUpdate(DocumentEvent e)
      {
        developer.getParent().getParentModel().modified();
      }
      public void changedUpdate(DocumentEvent e)
      {
        developer.getParent().getParentModel().modified();
      }
    };

    name.setText(developer.getName());
    name.getDocument().addDocumentListener(new DocumentListener()
    {
      public void insertUpdate(DocumentEvent e)
      {
        developer.setName(name.getText());
      }

      public void removeUpdate(DocumentEvent e)
      {
        developer.setName(name.getText());
      }

      public void changedUpdate(DocumentEvent e)
      {
        developer.setName(name.getText());
      }
    });

    email.setText(developer.getEmail());
    email.getDocument().addDocumentListener(modificationNotifier);
    email.addFocusListener(new FocusAdapter()
    {
      @Override
      public void focusLost(FocusEvent e)
      {
        try
        {
          developer.setEmail(email.getText());
        }
        catch (Exception e1)
        {
          UIs.createMessagePopup(null, "invalid email address", Icons.ERROR).showUnderneathOf(email);
        }
      }
    });

    talkUsername.setText(developer.getTalkUsername());
    talkUsername.getDocument().addDocumentListener(modificationNotifier);
    talkUsername.addFocusListener(new FocusAdapter()
    {
      @Override
      public void focusLost(FocusEvent e)
      {
        developer.setTalkUsername(talkUsername.getText());
      }
    });

    repoUsername.setText(developer.getUserNames(Repository.Type.VERSION_CONTROL));
    repoUsername.getDocument().addDocumentListener(modificationNotifier);
    repoUsername.addFocusListener(new FocusAdapter()
    {
      @Override
      public void focusLost(FocusEvent e)
      {
        developer.setUserNames(Repository.Type.VERSION_CONTROL, repoUsername.getText());
      }
    });

    setVcsName(developer.getParent().getRepoName(Repository.Type.VERSION_CONTROL));
  }

  public void setVcsName(String vcsName)
  {
    if (vcsName == null || vcsName.length() == 0)
    {
      vcsName = repoDefaultName;
    }

    repoLabel.setText(repoPlaceholder.matcher(repoLabelContent).replaceAll(vcsName));
    repoLabel.setDisplayedMnemonic('u');
    repoLabel.setDisplayedMnemonicIndex(repoLabel.getText().lastIndexOf('u'));
    repoDescription.setText(repoPlaceholder.matcher(repoDescriptionContent).replaceAll(vcsName));
  }

  private void createUIComponents()
  {
    thisPanel = this;
    name = new com.olympya.tools.grok.intellij.ui.TextField();
    email = new com.olympya.tools.grok.intellij.ui.TextField();
    talkUsername = new com.olympya.tools.grok.intellij.ui.TextField();
    repoUsername = new com.olympya.tools.grok.intellij.ui.TextField();
    repoLabel = new JLabel();
    repoDescription = new JTextArea();
    setVcsName(null);
  }

  /**
   * Method generated by IntelliJ IDEA GUI Designer
   * >>> IMPORTANT!! <<<
   * DO NOT edit this method OR call it in your code!
   *
   * @noinspection ALL
   */
  private void $$$setupUI$$$()
  {
    createUIComponents();
    thisPanel.setLayout(new GridLayoutManager(1, 1, new Insets(0, 0, 0, 0), -1, -1));
    final JPanel panel1 = new JPanel();
    panel1.setLayout(new GridLayoutManager(17, 1, new Insets(10, 10, 10, 10), -1, -1));
    thisPanel.add(panel1, new GridConstraints(0, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_VERTICAL, GridConstraints.SIZEPOLICY_FIXED, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, new Dimension(330, -1), null, new Dimension(330, -1), 0, false));
    final JLabel label1 = new JLabel();
    label1.setFont(new Font(label1.getFont().getName(), Font.BOLD, label1.getFont().getSize()));
    label1.setText("Name:");
    label1.setDisplayedMnemonic('N');
    label1.setDisplayedMnemonicIndex(0);
    panel1.add(label1, new GridConstraints(0, 0, 1, 1, GridConstraints.ANCHOR_WEST, GridConstraints.FILL_NONE, GridConstraints.SIZEPOLICY_FIXED, GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
    panel1.add(name, new GridConstraints(2, 0, 1, 1, GridConstraints.ANCHOR_WEST, GridConstraints.FILL_NONE, GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_FIXED, null, new Dimension(250, -1), new Dimension(250, -1), 0, false));
    final Spacer spacer1 = new Spacer();
    panel1.add(spacer1, new GridConstraints(3, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_VERTICAL, 1, GridConstraints.SIZEPOLICY_FIXED, null, new Dimension(-1, 20), null, 0, false));
    final JLabel label2 = new JLabel();
    label2.setFont(new Font(label2.getFont().getName(), Font.BOLD, label2.getFont().getSize()));
    label2.setText("E-mail:");
    label2.setDisplayedMnemonic('E');
    label2.setDisplayedMnemonicIndex(0);
    panel1.add(label2, new GridConstraints(4, 0, 1, 1, GridConstraints.ANCHOR_WEST, GridConstraints.FILL_NONE, GridConstraints.SIZEPOLICY_FIXED, GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
    panel1.add(email, new GridConstraints(6, 0, 1, 1, GridConstraints.ANCHOR_WEST, GridConstraints.FILL_NONE, GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_FIXED, null, new Dimension(250, -1), new Dimension(250, -1), 0, false));
    final Spacer spacer2 = new Spacer();
    panel1.add(spacer2, new GridConstraints(7, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_VERTICAL, 1, GridConstraints.SIZEPOLICY_FIXED, null, new Dimension(-1, 20), null, 0, false));
    final Spacer spacer3 = new Spacer();
    panel1.add(spacer3, new GridConstraints(16, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_VERTICAL, 1, GridConstraints.SIZEPOLICY_WANT_GROW, null, null, null, 0, false));
    final Spacer spacer4 = new Spacer();
    panel1.add(spacer4, new GridConstraints(11, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_VERTICAL, 1, GridConstraints.SIZEPOLICY_FIXED, null, new Dimension(-1, 20), null, 0, false));
    repoLabel.setFont(new Font(repoLabel.getFont().getName(), Font.BOLD, repoLabel.getFont().getSize()));
    panel1.add(repoLabel, new GridConstraints(12, 0, 1, 1, GridConstraints.ANCHOR_WEST, GridConstraints.FILL_NONE, GridConstraints.SIZEPOLICY_FIXED, GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
    panel1.add(repoUsername, new GridConstraints(14, 0, 1, 1, GridConstraints.ANCHOR_WEST, GridConstraints.FILL_NONE, GridConstraints.SIZEPOLICY_WANT_GROW, GridConstraints.SIZEPOLICY_FIXED, null, new Dimension(250, -1), new Dimension(250, -1), 0, false));
    final JLabel label3 = new JLabel();
    label3.setFont(new Font(label3.getFont().getName(), Font.BOLD, label3.getFont().getSize()));
    label3.setText("IDEtalk username");
    label3.setDisplayedMnemonic('T');
    label3.setDisplayedMnemonicIndex(3);
    panel1.add(label3, new GridConstraints(8, 0, 1, 1, GridConstraints.ANCHOR_WEST, GridConstraints.FILL_NONE, GridConstraints.SIZEPOLICY_FIXED, GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
    panel1.add(talkUsername, new GridConstraints(10, 0, 1, 1, GridConstraints.ANCHOR_WEST, GridConstraints.FILL_NONE, GridConstraints.SIZEPOLICY_WANT_GROW, GridConstraints.SIZEPOLICY_FIXED, null, new Dimension(250, -1), new Dimension(250, -1), 0, false));
    final JTextArea textArea1 = new JTextArea();
    textArea1.setDisabledTextColor(UIManager.getColor("Label.foreground"));
    textArea1.setEnabled(false);
    textArea1.setFont(UIManager.getFont("Label.font"));
    textArea1.setLineWrap(true);
    textArea1.setText("Name explanation");
    textArea1.setWrapStyleWord(true);
    panel1.add(textArea1, new GridConstraints(1, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_BOTH, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, null, null, null, 0, false));
    final JTextArea textArea2 = new JTextArea();
    textArea2.setDisabledTextColor(UIManager.getColor("Label.foreground"));
    textArea2.setEnabled(false);
    textArea2.setFont(UIManager.getFont("Label.font"));
    textArea2.setLineWrap(true);
    textArea2.setText("IDEtalk explanation");
    textArea2.setWrapStyleWord(true);
    panel1.add(textArea2, new GridConstraints(9, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_BOTH, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, null, null, null, 0, false));
    repoDescription.setDisabledTextColor(UIManager.getColor("Label.foreground"));
    repoDescription.setEnabled(false);
    repoDescription.setFont(UIManager.getFont("Label.font"));
    repoDescription.setLineWrap(true);
    repoDescription.setWrapStyleWord(true);
    panel1.add(repoDescription, new GridConstraints(13, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_BOTH, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, null, null, null, 0, false));
    final JTextArea textArea3 = new JTextArea();
    textArea3.setDisabledTextColor(UIManager.getColor("Label.foreground"));
    textArea3.setEnabled(false);
    textArea3.setFont(UIManager.getFont("Label.font"));
    textArea3.setLineWrap(true);
    textArea3.setText("E-mail explanation");
    textArea3.setWrapStyleWord(true);
    panel1.add(textArea3, new GridConstraints(5, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_BOTH, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, null, null, null, 0, false));
    final Spacer spacer5 = new Spacer();
    panel1.add(spacer5, new GridConstraints(15, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_VERTICAL, 1, GridConstraints.SIZEPOLICY_FIXED, null, new Dimension(-1, 20), null, 0, false));
    label1.setLabelFor(name);
    label2.setLabelFor(email);
    repoLabel.setLabelFor(repoUsername);
    label3.setLabelFor(talkUsername);
  }

  /**
   * @noinspection ALL
   */
  public JComponent $$$getRootComponent$$$()
  {
    return thisPanel;
  }
}