package com.olympya.tools.grok.intellij.ui.conf;

import javax.swing.tree.*;

import com.olympya.tools.grok.*;

/**
 * A node representing a developer.
 *
 * @author Hugo Baés
 */
class DeveloperTreeNode
        implements Comparable<DeveloperTreeNode>
{
  private WorkgroupTreeNode parent;
  private final Developer.Builder builder;

  /**
   * New blank developer node.
   */
  public DeveloperTreeNode(WorkgroupTreeNode parent)
  {
    if (parent == null)
      throw new NullPointerException("parent");

    this.parent = parent;
    builder     = new Developer.Builder().name("New Developer");
  }

  /**
   * Creates a node representation from the specified developer.
   */
  public DeveloperTreeNode(WorkgroupTreeNode parent, Developer developer)
  {
    if (parent == null)
      throw new NullPointerException("parent");

    if (developer == null)
      throw new NullPointerException("developer");

    this.parent = parent;
    builder     = new Developer.Builder(developer);
  }

  public String getName()
  {
    return builder.name();
  }

  public void setName(String name)
  {
    if (builder.name().equals(name))
      return;

    builder.name(name);
    parent.getParentModel().modified();
    parent.getParentModel().valueForPathChanged(getPath(), this);
  }

  public String getEmail()
  {
    return builder.email();
  }

  public void setEmail(String email)
  {
    if (builder.email().equals(email))
      return;

    builder.email(email);
    parent.getParentModel().modified();
  }

  public String getTalkUsername()
  {
    return builder.im();
  }

  public void setTalkUsername(String talkUsername)
  {
    if (builder.im().equals(talkUsername))
      return;

    builder.im(talkUsername);
    parent.getParentModel().modified();
  }

  public String getUserNames(Repository.Type type)
  {
    return builder.userNames().get(type);
  }

  public void setUserNames(Repository.Type type, String userName)
  {
    if (builder.userNames().get(type) != null && builder.userNames().get(type).equals(userName))
      return;

    builder.userNames(type, userName);
    parent.getParentModel().modified();
  }

  /**
   * Builds a new developer from the node information.
   */
  public Developer toDeveloper()
  {
    return builder.build();
  }

  /**
   * Returns the path to this developer.
   */
  public TreePath getPath()
  {
    if (parent != null)
      return parent.getPath().pathByAddingChild(this);

    return new TreePath(this);
  }

  /**
   * Used only by its parent workgroup.
   */
  void setParent(WorkgroupTreeNode workgroup)
  {
    this.parent = workgroup;
  }

  /**
   * Returns the workgroup that contains this developer.
   */
  public WorkgroupTreeNode getParent()
  {
    return parent;
  }

  @Override
  public String toString()
  {
    return getName();
  }

  public int compareTo(DeveloperTreeNode o)
  {
    return builder.name().compareToIgnoreCase(o.getName());
  }
}
