package com.olympya.tools.grok.intellij.ui.conf;

import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.util.*;
import java.util.List;
import javax.swing.*;
import javax.swing.event.*;

import static com.google.common.collect.Lists.*;
import com.intellij.openapi.project.*;
import com.intellij.openapi.vcs.*;
import com.intellij.uiDesigner.core.*;

import com.olympya.tools.grok.*;
import com.olympya.tools.grok.intellij.*;
import com.olympya.tools.grok.intellij.ui.*;
import com.olympya.tools.grok.metrics.*;

/**
 * Panel for Workgroup Settings.
 *
 * @author Hugo Baés
 */
class WorkgroupPanel
        extends JPanel
{
  // Package-private components
  JTextField name;
  JComboBox metrics;
  JComboBox vcsName;
  JTextField vcsRoot;
  JComboBox issueTracker;
  JTextField issueUrl;

  private JPanel thisPanel;
  private JTabbedPane tab;
  private JTextArea metricDescription;


  WorkgroupPanel(final WorkgroupTreeNode workgroup)
  {
    $$$setupUI$$$();

    tab.setIconAt(0, Icons.SETTINGS);
    tab.setIconAt(1, Icons.VCS);

    DocumentListener modificationNotifier = new DocumentListener()
    {
      public void insertUpdate(DocumentEvent e)
      {
        workgroup.getParentModel().modified();
      }
      public void removeUpdate(DocumentEvent e)
      {
        workgroup.getParentModel().modified();
      }
      public void changedUpdate(DocumentEvent e)
      {
        workgroup.getParentModel().modified();
      }
    };

    name.setText(workgroup.getName());
    name.getDocument().addDocumentListener(new DocumentListener()
    {
      public void insertUpdate(DocumentEvent e)
      {
        workgroup.setName(name.getText());
      }
      public void removeUpdate(DocumentEvent e)
      {
        workgroup.setName(name.getText());
      }
      public void changedUpdate(DocumentEvent e)
      {
        workgroup.setName(name.getText());
      }
    });

    // Fill metric combo box
    metrics.removeAllItems();
    List<Metric> metricList = newArrayList(Workgroup.defaultMetrics());
    Collections.reverse(metricList);
    for (Metric metric : metricList)
    {
      metrics.addItem(metric);
    }
    Metric metric = workgroup.getMetric(Repository.Type.VERSION_CONTROL);
    metrics.setSelectedItem(metric);
    metricDescription.setText(metric != null ? metric.getDescription() : null);
    metrics.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        Metric metric = (Metric) metrics.getSelectedItem();
        workgroup.setMetric(Repository.Type.VERSION_CONTROL, metric);
        metricDescription.setText(metric.getDescription());
      }
    });

    // Fill version control combo box
    vcsName.removeAllItems();
    Project defaultProject = ProjectManager.getInstance().getDefaultProject();
    ProjectLevelVcsManager vcsManager = ProjectLevelVcsManager.getInstance(defaultProject);
    for (AbstractVcs vcs : vcsManager.getAllVcss())
    {
      vcsName.addItem(vcs.getDisplayName());
    }
    vcsName.setSelectedItem(workgroup.getRepoName(Repository.Type.VERSION_CONTROL));
    vcsName.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        workgroup.setRepoName(Repository.Type.VERSION_CONTROL, (String) vcsName.getSelectedItem());
      }
    });

    URI repoRoot = workgroup.getRepoRoot(Repository.Type.VERSION_CONTROL);
    vcsRoot.setText(repoRoot != null ? repoRoot.toString() : null);
    vcsRoot.getDocument().addDocumentListener(modificationNotifier);
    vcsRoot.addFocusListener(new FocusAdapter()
    {
      @Override
      public void focusLost(FocusEvent e)
      {
        try
        {
          URI uri = URI.create(vcsRoot.getText());
          if (uri.getScheme() == null)
          {
            uri = URI.create("http://" + uri.toString());
            vcsRoot.setText(uri.toString());
          }
          workgroup.setRepoRoot(Repository.Type.VERSION_CONTROL, uri);
        }
        catch (Exception e1)
        {
          UIs.createMessagePopup(null, "invalid URL root", Icons.ERROR).showUnderneathOf(vcsRoot);
        }
      }
    });

    // Fill issue tracker combo box
    issueTracker.removeAllItems();
    for (AbstractIssueTracker tracker : AbstractIssueTracker.getAllIssueTrackers())
    {
      issueTracker.addItem(tracker.getDisplayName());
    }
    issueTracker.setSelectedItem(workgroup.getRepoName(Repository.Type.ISSUE_TRACKER));
    issueTracker.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        workgroup.setRepoName(Repository.Type.ISSUE_TRACKER, (String) issueTracker.getSelectedItem());
      }
    });

    URI issueRoot = workgroup.getRepoRoot(Repository.Type.ISSUE_TRACKER);
    issueUrl.setText(issueRoot != null ? issueRoot.toString() : null);
    issueUrl.getDocument().addDocumentListener(modificationNotifier);
    issueUrl.addFocusListener(new FocusAdapter()
    {
      @Override
      public void focusLost(FocusEvent e)
      {
        try
        {
          URI uri = URI.create(issueUrl.getText());
          if (uri.getScheme() == null)
          {
            uri = URI.create("http://" + uri.toString());
            issueUrl.setText(uri.toString());
          }

          workgroup.setRepoRoot(Repository.Type.ISSUE_TRACKER, uri);
        }
        catch (Exception e1)
        {
          UIs.createMessagePopup(null, "invalid URL root", Icons.ERROR).showUnderneathOf(issueUrl);
        }
      }
    });
  }

  private void createUIComponents()
  {
    thisPanel = this;
    name = new com.olympya.tools.grok.intellij.ui.TextField();
    metrics = new ComboBox();
    vcsName = new ComboBox();
    vcsRoot = new com.olympya.tools.grok.intellij.ui.TextField();
    issueTracker = new ComboBox();
    issueUrl = new com.olympya.tools.grok.intellij.ui.TextField();
  }

  /**
   * Method generated by IntelliJ IDEA GUI Designer
   * >>> IMPORTANT!! <<<
   * DO NOT edit this method OR call it in your code!
   *
   * @noinspection ALL
   */
  private void $$$setupUI$$$()
  {
    createUIComponents();
    thisPanel.setLayout(new GridLayoutManager(1, 1, new Insets(0, 0, 0, 0), -1, -1));
    tab = new JTabbedPane();
    thisPanel.add(tab, new GridConstraints(0, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_VERTICAL, GridConstraints.SIZEPOLICY_FIXED, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, new Dimension(330, -1), new Dimension(200, 200), new Dimension(330, -1), 0, false));
    final JPanel panel1 = new JPanel();
    panel1.setLayout(new GridLayoutManager(9, 1, new Insets(10, 10, 10, 10), -1, -1));
    tab.addTab("Profile", panel1);
    final JLabel label1 = new JLabel();
    label1.setFont(new Font(label1.getFont().getName(), Font.BOLD, label1.getFont().getSize()));
    label1.setText("Name:");
    label1.setDisplayedMnemonic('N');
    label1.setDisplayedMnemonicIndex(0);
    panel1.add(label1, new GridConstraints(0, 0, 1, 1, GridConstraints.ANCHOR_WEST, GridConstraints.FILL_NONE, GridConstraints.SIZEPOLICY_FIXED, GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
    final JTextArea textArea1 = new JTextArea();
    textArea1.setDisabledTextColor(UIManager.getColor("Label.foreground"));
    textArea1.setEnabled(false);
    textArea1.setFont(UIManager.getFont("Label.font"));
    textArea1.setLineWrap(true);
    textArea1.setText("Name explanation");
    textArea1.setWrapStyleWord(true);
    panel1.add(textArea1, new GridConstraints(1, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_BOTH, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, null, null, null, 0, false));
    panel1.add(name, new GridConstraints(2, 0, 1, 1, GridConstraints.ANCHOR_WEST, GridConstraints.FILL_NONE, GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_FIXED, null, new Dimension(250, -1), new Dimension(250, -1), 0, false));
    final Spacer spacer1 = new Spacer();
    panel1.add(spacer1, new GridConstraints(8, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_VERTICAL, 1, GridConstraints.SIZEPOLICY_WANT_GROW, new Dimension(20, -1), null, null, 0, false));
    final Spacer spacer2 = new Spacer();
    panel1.add(spacer2, new GridConstraints(3, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_VERTICAL, 1, GridConstraints.SIZEPOLICY_FIXED, null, new Dimension(-1, 20), null, 0, false));
    final JLabel label2 = new JLabel();
    label2.setFont(new Font(label2.getFont().getName(), Font.BOLD, label2.getFont().getSize()));
    label2.setText("Metric:");
    label2.setDisplayedMnemonic('M');
    label2.setDisplayedMnemonicIndex(0);
    panel1.add(label2, new GridConstraints(4, 0, 1, 1, GridConstraints.ANCHOR_WEST, GridConstraints.FILL_NONE, GridConstraints.SIZEPOLICY_FIXED, GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
    final JTextArea textArea2 = new JTextArea();
    textArea2.setDisabledTextColor(UIManager.getColor("Label.foreground"));
    textArea2.setEnabled(false);
    textArea2.setFont(UIManager.getFont("Label.font"));
    textArea2.setLineWrap(true);
    textArea2.setText("Metric explanation");
    textArea2.setWrapStyleWord(true);
    panel1.add(textArea2, new GridConstraints(5, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_BOTH, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, null, null, null, 0, false));
    panel1.add(metrics, new GridConstraints(6, 0, 1, 1, GridConstraints.ANCHOR_WEST, GridConstraints.FILL_NONE, GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_FIXED, null, new Dimension(250, -1), new Dimension(250, -1), 0, false));
    metricDescription = new JTextArea();
    metricDescription.setDisabledTextColor(UIManager.getColor("Label.foreground"));
    metricDescription.setEnabled(false);
    metricDescription.setFont(UIManager.getFont("Label.font"));
    metricDescription.setLineWrap(true);
    metricDescription.setWrapStyleWord(true);
    panel1.add(metricDescription, new GridConstraints(7, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_BOTH, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, null, null, null, 0, false));
    final JPanel panel2 = new JPanel();
    panel2.setLayout(new GridLayoutManager(17, 1, new Insets(10, 10, 10, 10), -1, -1));
    tab.addTab("Repositories", panel2);
    final Spacer spacer3 = new Spacer();
    panel2.add(spacer3, new GridConstraints(16, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_VERTICAL, 1, GridConstraints.SIZEPOLICY_WANT_GROW, null, null, null, 0, false));
    final JLabel label3 = new JLabel();
    label3.setFont(new Font(label3.getFont().getName(), Font.BOLD, label3.getFont().getSize()));
    label3.setText("Version Control System");
    label3.setDisplayedMnemonic('V');
    label3.setDisplayedMnemonicIndex(0);
    panel2.add(label3, new GridConstraints(0, 0, 1, 1, GridConstraints.ANCHOR_WEST, GridConstraints.FILL_NONE, GridConstraints.SIZEPOLICY_FIXED, GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
    final JTextArea textArea3 = new JTextArea();
    textArea3.setDisabledTextColor(UIManager.getColor("Label.foreground"));
    textArea3.setEnabled(false);
    textArea3.setFont(UIManager.getFont("Label.font"));
    textArea3.setText("VCS explanation");
    panel2.add(textArea3, new GridConstraints(1, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_BOTH, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, null, null, null, 0, false));
    panel2.add(vcsName, new GridConstraints(2, 0, 1, 1, GridConstraints.ANCHOR_WEST, GridConstraints.FILL_NONE, GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_FIXED, null, new Dimension(250, -1), new Dimension(250, -1), 0, false));
    final Spacer spacer4 = new Spacer();
    panel2.add(spacer4, new GridConstraints(3, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_VERTICAL, 1, GridConstraints.SIZEPOLICY_FIXED, null, new Dimension(-1, 20), null, 0, false));
    final JLabel label4 = new JLabel();
    label4.setFont(new Font(label4.getFont().getName(), Font.BOLD, label4.getFont().getSize()));
    label4.setText("Version Control Root");
    label4.setDisplayedMnemonic('R');
    label4.setDisplayedMnemonicIndex(16);
    panel2.add(label4, new GridConstraints(4, 0, 1, 1, GridConstraints.ANCHOR_WEST, GridConstraints.FILL_NONE, GridConstraints.SIZEPOLICY_FIXED, GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
    final JTextArea textArea4 = new JTextArea();
    textArea4.setDisabledTextColor(UIManager.getColor("Label.foreground"));
    textArea4.setEnabled(false);
    textArea4.setFont(UIManager.getFont("Label.font"));
    textArea4.setText("Version Control Root explanation");
    panel2.add(textArea4, new GridConstraints(5, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_BOTH, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, null, null, null, 0, false));
    panel2.add(vcsRoot, new GridConstraints(6, 0, 1, 1, GridConstraints.ANCHOR_WEST, GridConstraints.FILL_NONE, GridConstraints.SIZEPOLICY_WANT_GROW, GridConstraints.SIZEPOLICY_FIXED, null, new Dimension(250, -1), new Dimension(250, -1), 0, false));
    final Spacer spacer5 = new Spacer();
    panel2.add(spacer5, new GridConstraints(7, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_VERTICAL, 1, GridConstraints.SIZEPOLICY_FIXED, null, new Dimension(-1, 20), null, 0, false));
    final JLabel label5 = new JLabel();
    label5.setFont(new Font(label5.getFont().getName(), Font.BOLD, label5.getFont().getSize()));
    label5.setText("Issue Tracker");
    label5.setDisplayedMnemonic('T');
    label5.setDisplayedMnemonicIndex(6);
    panel2.add(label5, new GridConstraints(8, 0, 1, 1, GridConstraints.ANCHOR_WEST, GridConstraints.FILL_NONE, GridConstraints.SIZEPOLICY_FIXED, GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
    final JTextArea textArea5 = new JTextArea();
    textArea5.setDisabledTextColor(UIManager.getColor("Label.foreground"));
    textArea5.setEnabled(false);
    textArea5.setFont(UIManager.getFont("Label.font"));
    textArea5.setText("Issue tracker explanation");
    panel2.add(textArea5, new GridConstraints(9, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_BOTH, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, null, null, null, 0, false));
    panel2.add(issueTracker, new GridConstraints(10, 0, 1, 1, GridConstraints.ANCHOR_WEST, GridConstraints.FILL_NONE, GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_FIXED, null, new Dimension(250, -1), new Dimension(250, -1), 0, false));
    final Spacer spacer6 = new Spacer();
    panel2.add(spacer6, new GridConstraints(11, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_VERTICAL, 1, GridConstraints.SIZEPOLICY_FIXED, null, new Dimension(-1, 20), null, 0, false));
    final JLabel label6 = new JLabel();
    label6.setFont(new Font(label6.getFont().getName(), Font.BOLD, label6.getFont().getSize()));
    label6.setText("Issue Tracker URL");
    label6.setDisplayedMnemonic('U');
    label6.setDisplayedMnemonicIndex(14);
    panel2.add(label6, new GridConstraints(12, 0, 1, 1, GridConstraints.ANCHOR_WEST, GridConstraints.FILL_NONE, GridConstraints.SIZEPOLICY_FIXED, GridConstraints.SIZEPOLICY_FIXED, null, null, null, 0, false));
    final JTextArea textArea6 = new JTextArea();
    textArea6.setDisabledTextColor(UIManager.getColor("Label.foreground"));
    textArea6.setEnabled(false);
    textArea6.setFont(UIManager.getFont("Label.font"));
    textArea6.setText("Issue Tracker URL explanation");
    panel2.add(textArea6, new GridConstraints(13, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_BOTH, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, GridConstraints.SIZEPOLICY_CAN_SHRINK | GridConstraints.SIZEPOLICY_CAN_GROW, null, null, null, 0, false));
    panel2.add(issueUrl, new GridConstraints(14, 0, 1, 1, GridConstraints.ANCHOR_WEST, GridConstraints.FILL_NONE, GridConstraints.SIZEPOLICY_WANT_GROW, GridConstraints.SIZEPOLICY_FIXED, null, new Dimension(250, -1), new Dimension(250, -1), 0, false));
    final Spacer spacer7 = new Spacer();
    panel2.add(spacer7, new GridConstraints(15, 0, 1, 1, GridConstraints.ANCHOR_CENTER, GridConstraints.FILL_VERTICAL, 1, GridConstraints.SIZEPOLICY_FIXED, null, new Dimension(-1, 20), null, 0, false));
    label1.setLabelFor(name);
    label2.setLabelFor(metrics);
  }

  /**
   * @noinspection ALL
   */
  public JComponent $$$getRootComponent$$$()
  {
    return thisPanel;
  }
}