package com.olympya.tools.grok.metrics;

import java.util.*;

import com.olympya.tools.grok.*;

/**
 * Computes <em>{@linkplain Expertise implementation expertise}</em> metrics.
 *
 * @author Paulo Mattos
 */
public abstract class Metric
{
  /**
   * Constructor designed for subclassing.
   */
  protected Metric()
  {
  }

  /**
   * Returns the <em>implementation expertise</em> (i.e., <em>grokness</em> factor) of each developer, in the provided environment,
   * over the specified artifact. The returned map order is undefined.
   *
   * @see #sorted()
   */
  public abstract Map<Developer, Expertise> evaluate(Team team, Artifact artifact);

  /**
   * Returns a long textual description of this metric.
   */
  public abstract String getDescription();

  /**
   * Returns a short textual description of this metric.
   */
  public abstract String toString();

  //
  // Views
  //

  /**
   * Returns a <em>view</em> of this metric that <em>sorts</em> the returned
   * {@link #evaluate(Team, Artifact) Map<Developer, Expertise>} in <em>descending</em> order
   * by the {@code Expertise} value.
   */
  public Metric sorted()
  {
    return new SortedMetric(this);
  }

  /**
   * Returns a <em>view</em> of this metric that <em>normalizes</em> the returned
   * {@link #evaluate(Team, Artifact) Map<Developer, Expertise>} (e.g., the first developer
   * would achieve a relative 100% expertise level).
   */
  public Metric relativized()
  {
    return new RelativizedMetric(this);
  }

  /**
   * Returns a <em>view</em> of this metric that prints <em>timing statistics</em> to
   * the standard output.
   */
  public Metric timed()
  {
    return new TimedMetric(this);
  }
}
