package com.olympya.tools.grok.metrics;

import java.util.*;

import com.olympya.tools.grok.*;

/**
 * An abstract metric decorator (i.e., view).
 *
 * @author Paulo Mattos
 */
abstract class MetricDecorator
extends Metric
{
  private final Metric decorated;

  MetricDecorator(Metric decorated)
  {
    assert decorated != null;
    this.decorated = decorated;
  }

  /**
   * {@inheritDoc}
   */
  public Map<Developer, Expertise> evaluate(Team team, Artifact artifact)
  {
    return decorated.evaluate(team, artifact);
  }

  /**
   * {@inheritDoc}
   */
  public String getDescription()
  {
    return decorated.getDescription();
  }

  /**
   * {@inheritDoc}
   */
  public String toString()
  {
    return decorated.toString();
  }

  static void check(Map<Developer, Expertise> experts)
  {
    for (Map.Entry<Developer, Expertise> entry : experts.entrySet())
      if (entry.getKey() == null || entry.getValue() == null)
        throw new IllegalStateException("experts=" + experts);
  }
}
