package com.olympya.tools.grok.metrics;

import java.util.*;

import static com.google.common.collect.Maps.newLinkedHashMap;

import com.olympya.tools.grok.*;

/**
 * Implementation for {@link Metric#relativized()} view.
 *
 * @author Paulo Mattos
 */
final class RelativizedMetric
extends MetricDecorator
{
  RelativizedMetric(Metric decorated)
  {
    super(decorated);
  }

  @Override
  public Map<Developer, Expertise> evaluate(Team team, Artifact artifact)
  {
    return relativize(super.evaluate(team, artifact));
  }

  private static Map<Developer, Expertise> relativize(Map<Developer, Expertise> experts)
  {
    check(experts);

    if (experts.size() == 0)
      return experts;

    final float maxExpertise = experts.values().iterator().next().normalizedValue();

    // we must use a "insertion order" preserving map
    Map<Developer, Expertise> relativized = newLinkedHashMap();

    for (Map.Entry<Developer, Expertise> entry : experts.entrySet())
    {
      Expertise relative = Expertise.clamped(entry.getValue().normalizedValue() / maxExpertise);
      relativized.put(entry.getKey(), relative);
    }

    assert relativized.size() == experts.size();
    return relativized;
  }
}
