package com.olympya.tools.grok.metrics;

import java.util.*;
import static java.util.Collections.*;

import static com.google.common.collect.Maps.*;
import static com.google.common.collect.Lists.*;

import com.olympya.tools.grok.*;

/**
 * Implementation for {@link Metric#sorted()} view.
 *
 * @author Paulo Mattos
 */
final class SortedMetric
extends MetricDecorator
{
  private static final Comparator<Map.Entry<Developer, Expertise>> DEV_EXPERTISE_CMP;

  static
  {
    DEV_EXPERTISE_CMP = new Comparator<Map.Entry<Developer, Expertise>>()
    {
      public int compare(Map.Entry<Developer, Expertise> e1, Map.Entry<Developer, Expertise> e2)
      {
        return e1.getValue().compareTo(e2.getValue());
      }
    };
  }

  SortedMetric(Metric decorated)
  {
    super(decorated);
  }

  @Override
  public Map<Developer, Expertise> evaluate(Team team, Artifact artifact)
  {
    return sort(super.evaluate(team, artifact));
  }

  /**
   * Returns a sorted copy of the specified map in descending order by the expertise value.
   *
   * @throws IllegalStateException if any key or value of the specified map is {@code null}.
   */
  private static Map<Developer, Expertise> sort(Map<Developer, Expertise> experts)
  {
    check(experts);

    List<Map.Entry<Developer, Expertise>> entries;

    entries = newArrayList(experts.entrySet());
    Collections.sort(entries, reverseOrder(DEV_EXPERTISE_CMP));

    // we must use a "insertion order" preserving map
    Map<Developer, Expertise> sortedExperts = newLinkedHashMap();

    for (Map.Entry<Developer, Expertise> entry : entries)
      sortedExperts.put(entry.getKey(), entry.getValue());

    assert experts.size() == sortedExperts.size();
    return sortedExperts;
  }
}
