package com.olympya.util.xml;

import java.util.*;
import javax.xml.bind.annotation.adapters.*;

/**
 * A extensible {@link EnumMap} adapter compatible with the {@link XmlJavaTypeAdapter} annotation.
 *
 * @param <K> the enum type of keys maintained by the <em>bound</em> map.
 * @param <V> the type of values maintained by the <em>bound</em> map.
 *
 * @author Paulo Mattos
 */
public abstract class XmlEnumMapAdapter<K extends Enum<K>, V>
extends XmlMapAdapter<K, V>
{
  private final Class<K> enumType;

  /**
   *
   */
  protected XmlEnumMapAdapter(Class<K> enumType, boolean immutable)
  {
    super(immutable);

    if (enumType == null)
      throw new NullPointerException("enumType");

    this.enumType = enumType;
  }

  /**
   * {@inheritDoc}
   */
  protected String marshalKey(K enumKey)
  {
    assert enumKey != null;
    return enumKey.name();
  }

  /**
   * {@inheritDoc}
   */
  protected K unmarshalKey(String enumName)
  {
    assert enumName != null;
    return Enum.valueOf(enumType, enumName);
  }
}
